/*
 * Spielplatz_90CAN.c
 *
 * Created: 22.02.2016 13:42:49
 *  Author: grriedel
 */

#include <avr/io.h>         // Zugriff auf IO's des Controllers
#include <avr/interrupt.h>	// Zugriff auf Interrupts
#include <avr/sleep.h>

#define F_CPU 16000000UL

#include <util/delay.h>

#include "FT800.h"
#include "FT800_commands.h"
#include "pic_data.h"



volatile uint8_t roundtrip=0;

// wird alle 250s aufgerufen
ISR (TIMER1_COMPA_vect)
{
	roundtrip++;
	roundtrip &= 0x03; // limit to 4 time-slots
}


void init_timer1(void)
{
	// Timer1 - 250s @ 2 Mhz
	TCCR1B = 0x00; // stop
	TCCR1A = 0x00;
	TCCR1C = 0x00;
	OCR1A = 499; // TOP - timer zhlt bis TOP und wirft nen IRQ
	TIMSK1 |= (1<<OCIE1A);
	TCCR1B = (1<<WGM12) | (1<<CS11); // mode = CTC, prescaler = 8 -> 2 MHz
}


void init_ports(void)
{
	DDRA = 0x00;  // alle Pins auf Eingang
	PORTA = 0xff; // Pullups an

	DDRB = (1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB4) | (1<<PB5);  // SS (ungenutzt), SCK, MOSI, PB4=CS, PB5=PowerDown auf Ausgang, Rest auf Eingang
	PORTB = (1<<PB0) | (1<PB4); // SS (ungenutzt) auf High, CS auf High, PowerDown auf Low

	DDRC = 0x00;  // alle Pins auf Eingang
	PORTC = 0xff; // Pullups an

	DDRD = 0x00; // alle Pins auf Eingang
	PORTD = 0x9f; // Pullup-Widerstnde an

	DDRE = 0x00;  // alle Pins auf Eingang
	PORTE = 0xff; // Pullups an

	DDRF = 0x00;  // alle Pins auf Eingang
	PORTF = 0xfe; // Pullup-Widerstnde an

	DDRG = 0x00; // alle Pins auf Eingang
	PORTG = 0xff; // Pullup-Widerstnde an
}


/* CAN-Funktionen */

void init_can(void)
{
	uint8_t i,j;
	uint16_t id;

	CANGCON = (1<<SWRES); // off and reset

	//reset all MObs

	for (i=0; i<15; i++)
	{
		CANPAGE = (i<<4); // select MOb
		CANCDMOB = 0x00;		// disable MOb
		CANSTMOB = 0x00;		// clear status
		CANIDT1 = 0x00;		// clear ID
		CANIDT2 = 0x00;
		CANIDT3 = 0x00;
		CANIDT4 = 0x00;
		CANIDM1 = 0x00;		// clear mask
		CANIDM2 = 0x00;
		CANIDM3 = 0x00;
		CANIDM4 = 0x00;
		for (j=0; j<8; j++)
		{
			CANMSG = 0x00; // clear data
		}
	}

	// setup fr 500 kBit/s mit 16 MHz Quarz
	CANBT1 = (1<<BRP0);
	CANBT2 = (1<<PRS2) | (1<<PRS1);
	CANBT3 = (1<<PHS21) | (1<<PHS20) | (1<<PHS11) | (1<<PHS10) | (1<<SMP);

	CANGCON = (1<<ENASTB); // enable CAN

	id = 666;   // Sende-ID PWM
	CANPAGE = (0<<4); // select MOB0
	CANIDT1 = (id >> 3); // Put bits 3-11 of message ID into CANIDT1
	CANIDT2 = (id << 5); // Put bits 0-2 of message ID into CANIDT2
	// full filtering, only accept id given
	CANIDM1 = 0xff;
	CANIDM2 = 0xe0;
	// CANIDM3 = 0x00;
	CANIDM4 = (1<<IDEMSK);
	CANCDMOB = (1<<DLC3); // set to transmit, ext. ID, 8-byte in message
	CANSTMOB |= (1<<TXOK); // set flag

	CANGIE = 0x00;

	CANEN1 = 0x00;
	CANEN2 = (1<<ENMOB0); // enable MOB0

	CANIE1 = 0x00;

	CANSIT1 = 0x00;
	CANSIT2 = 0x00;
}

/* Ende CAN-Funktionen */


void init_spi(void)
{
	SPCR = (1<<SPE) | (1<<MSTR);	// SPI on, MSB first, Mode 0, Master, Fosc/4 -> 4MHz @ 16MHz CPU
	SPSR = (1<<SPI2X); // Fosc/2 -> 8 MHz @ 16MHz CPU
}


void ft800_loop()
{
	uint32_t calc;
	//	uint16_t xvalue,yvalue;
	//uint8_t pendown;
	static uint8_t tag=0, pic=0, pic2=0;
	static uint16_t button = 0;
	static uint16_t opt_keys=0;
//	static uint16_t clock_rad=25;
	static uint16_t dot_rad = 0;
	static uint8_t delay;
	static uint16_t progress = 0;
	static uint16_t toggle_state = 0;
	static uint8_t toggle_lock = 0;
	static int32_t rotate = 0;
	static int16_t x2 = 215;
	static uint8_t upx = 0;
	static uint16_t slider_val = 2048;
	static uint8_t track = 0;
	static uint32_t tracker_val = 0;
	static uint16_t scroller_val = 1024;
	static int16_t xpos_rel = 0;

	//	uint32_t touch_a, touch_b, touch_c, touch_d, touch_e, touch_f;

	//	calc = ft800_memRead32(REG_TOUCH_DIRECT_XY);
	//	yvalue = (calc & 0x03ff);//get the bits depending the register inf
	//	xvalue = ((calc >>16) & 0x03ff);
	//	pendown = ((calc & 0x8000) == 0) ? 0 : 1;


	//	button = (tag == 1) ? FT_OPT_FLAT : 0;
	//	pic = (tag == 4) ? 1 : 0;
	//	pic2 = (tag == 5) ? 1 : 0;


	//	touch_a = ft800_memRead32(REG_TOUCH_TRANSFORM_A);
	//	touch_b = ft800_memRead32(REG_TOUCH_TRANSFORM_B);
	//	touch_c = ft800_memRead32(REG_TOUCH_TRANSFORM_C);
	//	touch_d = ft800_memRead32(REG_TOUCH_TRANSFORM_D);
	//	touch_e = ft800_memRead32(REG_TOUCH_TRANSFORM_E);
	//	touch_f = ft800_memRead32(REG_TOUCH_TRANSFORM_F);

	//calc = cmdOffset;

		switch(delay)
		{
			case 0:
				calc = ft800_get_touch_tag();
				tag = calc;

				button = 0;
				pic = 0;
				pic2 = 0;
				opt_keys = 0;
//				clock_rad = 25;
				dot_rad = 0;

				switch(tag)
				{
					case 0:
						toggle_lock = 0;
						if(track != 0)
						{
							ft800_cmd_track(0, 0, 0, 0, 0); // stop tracking
							ft800_cmd_execute();
							track = 0;
						}

						break;
					case 1:
						button = FT_OPT_FLAT;
						break;
					case 4:
						pic = 1;
						break;
					case 5:
						pic2 = 1;
						break;
					case 6:
//						clock_rad = 45;
						break;
					case 8:
						if(toggle_lock == 0)
						{
							toggle_lock = 42;
							if(toggle_state == 0)
							{
								toggle_state = 0xffff;
							}
							else
							{
								toggle_state = 0;
							}
						}
						break;
					case 7:		// scroller
						if(track == 0)
						{
							ft800_cmd_track(5,190,85,10,7); // start tracking
							ft800_cmd_execute();
							track = 42;
						}
						else
						{

							tracker_val = ft800_memRead32(REG_TRACKER);
							if((tracker_val & 0xff) == 7)
							{
								tracker_val = (tracker_val >> 20);
								scroller_val = tracker_val;
							}
						}
						break;
					case 10:	// slider
						if(track == 0)
						{
							ft800_cmd_track(5,220,150,10,10); // start tracking
							ft800_cmd_execute();
							track = 42;
						}
						else
						{

							tracker_val = ft800_memRead32(REG_TRACKER);
							if((tracker_val & 0xff) == 10)
							{
								tracker_val = (tracker_val >> 20);
								slider_val = tracker_val;
								xpos_rel = tracker_val;
								xpos_rel /= 32;
								xpos_rel -= 64;
							}
						}
						break;
					case 49:
						opt_keys = '1';
						dot_rad = 10;
						break;
					case 50:
						opt_keys = '2';
						dot_rad = 15;
						break;
					case 51:
						opt_keys = '3';
						dot_rad = 20;
						break;
					case 52:
						opt_keys = '4';
						dot_rad = 20;
						break;
					case 53:
						opt_keys = '5';
						dot_rad = 25;
						break;
				}


			case 3:
/*
				ft800_cmd_dl(CMD_DLSTART); // Start the display list
				ft800_cmd_dl(CLEAR_COLOR_RGB(170,0,0)); // Set the default clear color to black
				ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); // Clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers
				ft800_cmd_dl(COLOR_RGB(0,56,112));
				ft800_cmd_bgcolor(0x00, 0x20, 0x40);
				ft800_cmd_gauge(149,110,50,0, 5,5,500,1000);
				ft800_cmd_dl(DL_DISPLAY);	// Instruct the graphics processor to show the list
				ft800_cmd_dl(CMD_SWAP); // Make this list active
				ft800_cmd_execute();
*/
				break;

			case 4:
				ft800_cmd_dl(CMD_DLSTART); // Start the display list
				ft800_cmd_dl(DL_CLEAR_RGB | BLACK); // Set the default clear color to black
				ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); // Clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers
				ft800_cmd_dl(TAG(0));
				break;

			case 5:
				ft800_cmd_text(xpos_rel+10, 20, 29, 0, "DING!");
//				ft800_cmd_text(230, 180, 26, 0, "1234567");
//				ft800_cmd_text(230, 195, 26, 0, "12345678");
//				ft800_cmd_text(230, 210, 26, 0, "123456789");
				break;

			case 6:
				ft800_cmd_text(xpos_rel, 0, 26, 0, "the quick brown fox jumps over the lazy dog");
				break;

			case 7:
				ft800_cmd_keys(xpos_rel+90, 25, 100, 20, 26, opt_keys, "12345");
				ft800_cmd_dl(DL_COLOR_RGB | ORANGE);
				ft800_cmd_dl(TAG(9));
				ft800_cmd_line(xpos_rel+215, 55, xpos_rel+310, 55, 1);
				ft800_cmd_line(xpos_rel+215, 55, xpos_rel+215, 100, 1);
				ft800_cmd_line(xpos_rel+215, 55, xpos_rel+x2, 100, 1);
				break;

			case 8:
				ft800_cmd_dl(TAG(1));
				ft800_cmd_dl(DL_COLOR_RGB | WHITE);
				ft800_cmd_fgcolor(0x101010);
				ft800_cmd_gradcolor(RED);
				ft800_cmd_button(xpos_rel+220, 25, 80, 20, 26, button, "Cat");
				ft800_cmd_fgcolor(0x003870); // back to default, light-blue
				break;

			case 9:
				ft800_cmd_dl(DL_COLOR_RGB | RED);
				ft800_cmd_dl(TAG(2));
				ft800_cmd_point(xpos_rel+260, 70, 5);

				ft800_cmd_dl(DL_COLOR_RGB | BLUE);
				ft800_cmd_dl(TAG(3));
				ft800_cmd_point(xpos_rel+280, 85, 8);

				if(dot_rad != 0)
				{
					ft800_cmd_dl(DL_COLOR_RGB | GREEN);
					ft800_cmd_point(xpos_rel+250, 100, dot_rad);
				}
				break;

			case 10:
				ft800_cmd_dl(DL_COLOR_RGB | WHITE);

				if(toggle_state == 0)
				{
					ft800_cmd_dl(TAG(4));
					ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
					ft800_cmd_dl(BITMAP_SOURCE(18432*pic));
					ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,96*2,96)); // jpg3
					ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,96,96)); // jpg3
					ft800_cmd_dl(VERTEX2F((xpos_rel+10)*16,50*16));
//					ft800_cmd_dl(VERTEX2II(xpos_rel+10,50,0,pic));
					ft800_cmd_dl(DL_END);
				}
				else
				{
					ft800_cmd_dl(TAG(0));
					ft800_cmd_dial(xpos_rel+40, 98, 32, 0, 0x8000);
				}

				ft800_cmd_dl(TAG(0));
				ft800_cmd_dl(DL_COLOR_RGB | PURPLE);
				ft800_cmd_rect(xpos_rel+120, 190, xpos_rel+180, 210, 3);
				ft800_cmd_dl(DL_COLOR_RGB | WHITE);
				break;

			case 11:
				if(toggle_state == 0)
				{
					ft800_cmd_dl(TAG(5));
					ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
					ft800_cmd_dl(BITMAP_SOURCE(40000));
					ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,80*2,105));
					ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,80,105));
					ft800_cmd_dl(VERTEX2II(xpos_rel+110,50,0,pic2));
					ft800_cmd_dl(DL_END);
				}
				else
				{
					ft800_cmd_dl(TAG(0));
					ft800_cmd_gauge(xpos_rel+120, 98, 40, 0, 5, 4, slider_val,4095);
				}
				break;

			case 12:

/*
				ft800_cmd_dl(TAG(0));
				ft800_cmd_dl(LINE_WIDTH(16));
				ft800_cmd_dl(DL_BEGIN | FT_LINE_STRIP);
				ft800_cmd_dl( VERTEX2F(220 * 16,180 * 16) );
				ft800_cmd_dl( VERTEX2F(260 * 16,180 * 16) );
				ft800_cmd_dl( VERTEX2F(270 * 16,190 * 16) );
				ft800_cmd_dl( VERTEX2F(230 * 16,190 * 16) );
				ft800_cmd_dl( VERTEX2F(220 * 16,180 * 16) );
				ft800_cmd_dl( VERTEX2F(220 * 16,220 * 16) );
				ft800_cmd_dl( VERTEX2F(260 * 16,220 * 16) );
				ft800_cmd_dl( VERTEX2F(260 * 16,180 * 16) );
				ft800_cmd_dl( VERTEX2F(270 * 16,190 * 16) );
				ft800_cmd_dl( VERTEX2F(270 * 16,230 * 16) );
				ft800_cmd_dl( VERTEX2F(230 * 16,230 * 16) );
				ft800_cmd_dl( VERTEX2F(230 * 16,190 * 16) );
				ft800_cmd_dl(DL_END);
*/

/*
				ft800_cmd_dl(TAG(0));
				ft800_cmd_dl(BITMAP_SOURCE(74000));
				ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,64*2,64));
				ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,64,64));
				ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
				ft800_cmd_dl(VERTEX2II(192,176,0,1));
				ft800_cmd_dl(VERTEX2II(256,112,0,2));
				ft800_cmd_dl(DL_END);
*/
/*
				ft800_cmd_dl(TAG(12));
				ft800_cmd_dl(DL_COLOR_RGB | RED);
				ft800_cmd_point(220,210, 20);
				ft800_cmd_dl(DL_COLOR_RGB | BLACK);
				ft800_cmd_point(220,210, 19);
				ft800_cmd_dl(DL_COLOR_RGB | WHITE);
				ft800_cmd_text(207, 203, 26, 0, "XxX");
*/
				//	ft800_cmd_number(60, 180, 26, 0, xvalue);
				//	ft800_cmd_number(100, 180, 26, 0, yvalue);
				//	ft800_cmd_number(140, 180, 26, 0, pendown);

				ft800_cmd_dl(TAG(0));
				ft800_cmd_text(xpos_rel+0, 150, 26, 0, "TAG:");
				ft800_cmd_number(xpos_rel+35, 150, 26, 0, tag);

	//	ft800_cmd_number(40, 180, 26, 0, (cmdOffset + 24) % 4);

	//	if(cmdOffset > calc)
	//	{
	//		cmd_use = (cmdOffset+24) - calc;
	//	}
	//	ft800_cmd_number(0, 210, 26, 0, cmd_use);

	//	ft800_cmd_number(0, 180, 26, 0, cmdOffset + 24);
				break;

			case 13:
//				ft800_cmd_dl(DL_COLOR_RGB | GREEN);
				ft800_cmd_progress(xpos_rel+5,170,85,10,0,progress,500);

				ft800_cmd_dl(TAG(7));
				ft800_cmd_scrollbar(xpos_rel+5,190,85,10,0,scroller_val,1024,4096);

				ft800_cmd_dl(TAG(10));
				ft800_cmd_slider(5,220,150,10,0,slider_val,4095);
				break;

			case 14:
				ft800_cmd_dl(TAG(8));
				ft800_cmd_toggle(xpos_rel+120, 165, 33, 26, 0, toggle_state, "aus" "\xff" "an");
//				ft800_cmd_dl(TAG(0));
//				ft800_cmd_number(5, 225, 26, 0, trackerval);
				break;

			case 15:
				ft800_cmd_dl(TAG(0));
				ft800_cmd_dl(BITMAP_SOURCE(74000));
				ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,64*2,64));
				ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,64,64));

				if(toggle_state == 0)
				{
					rotate = 0;
				}
				else
				{
					ft800_cmd_loadidentity();
					ft800_cmd_translate(65536 * 32, 65536 * 32);
					ft800_cmd_rotate(rotate);
					ft800_cmd_translate(65536 * -32, 65536 * -32);
					ft800_cmd_setmatrix();
					rotate += 256;
				}

				ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
				ft800_cmd_dl(VERTEX2II(xpos_rel+192,112,0,4));
				ft800_cmd_dl(DL_END);
				break;

			case 16:
				ft800_cmd_dl(TAG(0));
				ft800_cmd_dl(BITMAP_SOURCE(74000));
				ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,64*2,64));
				ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,64,64));

				if(toggle_state != 0) // x-flip an icon
				{
					ft800_cmd_loadidentity();
					ft800_cmd_translate(2097152, 2097152);
					ft800_cmd_scale(-65536, 65536);
					ft800_cmd_translate(-2097152, -2097152);
					ft800_cmd_setmatrix();
				}

				ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
				ft800_cmd_dl(VERTEX2II(xpos_rel+256,112,0,0));
				ft800_cmd_dl(DL_END);
				break;

			case 24:
				ft800_cmd_dl(DL_DISPLAY);	// Instruct the graphics processor to show the list
				ft800_cmd_dl(CMD_SWAP); // Make this list active
				ft800_cmd_execute();
				break;
		}

//		if(delay == 3)
//		{
//			delay = 25;
//		}

		delay++;
		if(delay > 24)
		{
			delay = 0;

			progress++;
			if(progress > 500)
			{
				progress = 0;
			}

			if(upx == 0)
			{
				x2+=1;
				if(x2 > 310)
				{
					x2 = 310;
					upx = 42;
				}
			}
			else
			{
				x2-=1;
				if(x2 < 215)
				{
					x2 = 215;
					upx = 0;
				}
			}
		}
}



int main(void)
{
	uint8_t triplock;

	cli();
	init_ports();
	init_timer1();
//	init_can();
	init_spi();
	sei();

	ft800_init();

	while(ft800_busy() == 1);
	ft800_get_cmdoffset();

	ft800_cmd_memset(0, 0xff,1L*96*96*2);	// Clear the memory at location 0 - any previous junk or bitmap data
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(0, FT_OPT_NODL, jpg3, 3708);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(18432, FT_OPT_NODL, jpg4, 2640);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(40000, FT_OPT_NODL, leopard, 3741);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(56800, FT_OPT_NODL, schneehase, 1833);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 0), FT_OPT_NODL, blitz, 1638);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 1), FT_OPT_NODL, figure, 2314);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 2), FT_OPT_NODL, frog, 1639);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 3), FT_OPT_NODL, map1, 2476);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(106768, FT_OPT_NODL, spiral, 2618);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	// send pre-recorded touch calibration values
	ft800_memWrite32(REG_TOUCH_TRANSFORM_A, 0xffffaa39);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_B, 0xffffffc6);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_C, 0x0149aab3);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_D, 0x0000001d);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_E, 0x000041c5);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_F, 0xfff675ba);

/*
	// calibrate touch
	ft800_cmd_dl(CMD_DLSTART); // Start the display list
	ft800_cmd_dl(DL_CLEAR_RGB | BLACK); // Set the default clear color to black
	ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); // Clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers
	ft800_cmd_text((FT_Width/2), (FT_Height/2), 27, FT_OPT_CENTER, "Please tap on the dot.");
	ft800_cmd_calibrate();
	ft800_cmd_dl(DL_DISPLAY);	// Instruct the graphics processor to show the list
	ft800_cmd_dl(CMD_SWAP); // Make this list active
	ft800_memWrite16(REG_CMD_WRITE, (cmdOffset));	// Update the ring buffer pointer so the graphics processor starts executing
*/
	while(ft800_busy() == 1);

	set_sleep_mode(SLEEP_MODE_IDLE); // einfach nur schlummern...

	while(1)
	{
		triplock = roundtrip;

		if(roundtrip == 0)
		{
			ft800_loop();
		}

		if(roundtrip == 1)
		{
		}

		if(roundtrip == 2)
		{
		}

		if(roundtrip == 3)
		{
		}

		while(triplock == roundtrip) // verhindern, das CAN IRQs den Controller wach machen
		{
			/* fr den Rest der Zeit-Scheibe schlafen legen */
			cli();
			sleep_enable();
			sei();
			sleep_cpu();
			sleep_disable();
		}
	} // while
}